// matrix class
// -Kyoung Shin Park (2007 Spring)
//////////////////////////////////////////////////////////////////////

#ifndef __MATRIX4x4_H__
#define __MATRIX4x4_H__

#include <cmath>
#include "vector3.h"

namespace oglclass {

#define DEGTORAD(degree) ((degree) * (3.141592654f / 180.0f))
#define RADTODEG(radian) ((radian) * (180.0f / 3.141592654f))

class matrix4x4
{
public:
    float m[16];

    matrix4x4()  { identity(); }

    matrix4x4( float m0, float m4, float  m8, float m12,
               float m1, float m5, float  m9, float m13,
               float m2, float m6, float m10, float m14,
               float m3, float m7, float m11, float m15 );

	// transformation matrix
    void identity(void);
    void translate(const vector3 &trans);
    void rotate(const float &angle, vector3 &axis);
    void rotate(const float &angle, unsigned char axis);
    void scale(const vector3 &scale);

    void transformPoint( vector3 *vec );
    void transformVector( vector3 *vec );

    // matrix operators...
    matrix4x4 operator + (const matrix4x4 &other);
    matrix4x4 operator - (const matrix4x4 &other);
    matrix4x4 operator * (const matrix4x4 &other);

    matrix4x4 operator * (const float scalar);
};

} // namespace

#endif
