// Room template
// -Kyoung Shin Park (2008 Spring)
//////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <stdio.h>      
#include <stdlib.h>     
#include <GL/gl.h>  
#include <GL/glu.h> 
#include <GL/glut.h>    
#include "simage.h"
#include "vector3.h"
#include "camera.h"

using namespace oglclass;

camera cam(60, 1, 0.1f, 1000.0f, FLY);
vector3 eye(100, 120, 50);
vector3 at(70, 80, 0);
vector3 up(0, 1, 0);

typedef struct _vertex
{
	float x, y, z;		// vertex
	float nx, ny, nz;	// normal
	float u, v;			// texture
} vertex;
vertex mesh[11][11];	// mesh
GLuint texture[3];		// storage for three texture
GLUquadricObj * quadric;

// init light
void initLight(void)
{
    GLfloat white[4] = { 1, 1, 1, 1 };
    GLfloat grey25[4] = { 0.25, 0.25, 0.25, 1 };
    GLfloat lightPos0[4] = { 0.5f, -0.5f, -0.5f, 0.0f };
    GLfloat lightPos1[4] = { -0.3f, -0.5f, 0.2f, 0.0f };

	glMaterialfv( GL_FRONT, GL_DIFFUSE, white);
	glMaterialfv( GL_FRONT, GL_AMBIENT, white);   

	// light0
	glLightfv( GL_LIGHT0, GL_DIFFUSE, white );
	glLightfv( GL_LIGHT0, GL_POSITION, lightPos0 );

	// light1
	glLightfv( GL_LIGHT1, GL_DIFFUSE, white );
	glLightfv( GL_LIGHT1, GL_POSITION, lightPos1 );

	glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHT1);

	// global ambient light
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, grey25);
}

// load texture
void initTexture(char *filename, GLuint& textureID, GLfloat wrap = GL_REPEAT)
{
	unsigned char *imageBuffer, *rescaledImage, *imagePtr;
	int imageWidth = 0, imageHeight = 0, numComponents = 0;
	imageBuffer = simage_read_image(filename, &imageWidth, &imageHeight, &numComponents);

	GLsizei xdim2,ydim2;

	xdim2 = 1;
    while (xdim2 <= imageWidth)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= imageHeight)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((imageWidth != xdim2) || (imageHeight != ydim2))
        {
        rescaledImage = simage_resize(imageBuffer, imageWidth, imageHeight, numComponents,
                                           xdim2, ydim2);
        imagePtr = rescaledImage;
        }
    else
        imagePtr = imageBuffer;

	if (imagePtr != NULL)
	{
		glGenTextures(1, &textureID);
		glBindTexture(GL_TEXTURE_2D, textureID);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, numComponents == 3 ? GL_RGB : GL_RGBA, imageWidth, imageHeight,
					 0, numComponents == 3 ? GL_RGB : GL_RGBA, GL_UNSIGNED_BYTE, imageBuffer);
		glBindTexture(GL_TEXTURE_2D, 0);
	}
}

// initialize mesh (NORMAL | VERTEX | TEX)
void initMesh()
{
	for(int i = 0;i <= 10; i++) { // row
		for(int j = 0;j <= 10; j++) { // column
			mesh[i][j].nx = 0;
			mesh[i][j].ny = 1;
			mesh[i][j].nz = 0;

			mesh[i][j].x = j*10;
			mesh[i][j].y = 0;
			mesh[i][j].z = -i*10;

			mesh[i][j].u = j*0.1;
			mesh[i][j].v = i*0.1;
		}
	}
}

// draw mesh
void drawMesh(GLuint texID)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texID);

	glBegin(GL_QUADS);
	for(int i = 0;i < 10; i++) {
		for(int j = 0;j < 10; j++) {
			glNormal3f(mesh[i][j].nx, mesh[i][j].ny, mesh[i][j].nz);

			glTexCoord2f(mesh[i][j].u, mesh[i][j].v);
			glVertex3f(mesh[i][j].x, mesh[i][j].y, mesh[i][j].z);

			glTexCoord2f(mesh[i][j+1].u, mesh[i][j+1].v);
			glVertex3f(mesh[i][j+1].x, mesh[i][j+1].y, mesh[i][j+1].z);

			glTexCoord2f(mesh[i+1][j+1].u, mesh[i+1][j+1].v);
			glVertex3f(mesh[i+1][j+1].x, mesh[i+1][j+1].y, mesh[i+1][j+1].z);

			glTexCoord2f(mesh[i+1][j].u, mesh[i+1][j].v);
			glVertex3f(mesh[i+1][j].x, mesh[i+1][j].y, mesh[i+1][j].z);
		}
	}
	glEnd();

	glDisable(GL_TEXTURE_2D);
}

void drawObjects(GLuint texID)
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texID);

	glPushMatrix();
	glTranslatef(50, 0, -50);
	glRotatef(-90, 1, 0, 0);
	gluCylinder(quadric, 10, 0, 50, 32, 16);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
}

void init(void) 
{
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f); // Clear The Background Color To Blue 
	glClearDepth(1.0);                    // Enables Clearing Of The Depth Buffer
	glDepthFunc(GL_LESS);                 // The Type Of Depth Test To Do
	glEnable(GL_DEPTH_TEST);              // Enables Depth Testing
	glShadeModel(GL_SMOOTH);              // Enables Smooth Color Shading
	glEnable(GL_TEXTURE_2D);              // Enable Texture Mapping

	initMesh();
	initLight();
	initTexture("greyBricks-lighter-512x512.jpg", texture[0]);
	initTexture("column_tan_256x512_03.jpg", texture[1]);
	initTexture("cobblestone_quad_01.jpg", texture[2]);

	cam.applyProjection();
	cam.lookAt(eye, at, up);
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// camera
	cam.apply();

	// floor
	drawMesh(texture[0]);

	// front face
	glPushMatrix();
	glTranslatef(0, 0, -100);
	glRotatef(90, 1, 0, 0);
	drawMesh(texture[1]);
	glPopMatrix();

	// left side
	glPushMatrix();
	glRotatef(90, 0, 1, 0);
	glRotatef(90, 1, 0, 0);
	drawMesh(texture[1]);
	glPopMatrix();

	// draw your mobile objects here!!
	drawObjects(texture[2]);

	glutSwapBuffers();                                          
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);                                 
	cam.setAspect((float)w / (float)h);
}

void keyboard(unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
	else if (key == ' ')
	{
		cam.lookAt(eye, at, up);
		printf("reset\n");
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
	if (key == GLUT_KEY_F1)			// x-movement
        cam.strafe(0.5);
	else if (key == GLUT_KEY_F2)
        cam.strafe(-0.5);
	else if (key == GLUT_KEY_F3)	// y-movement
        cam.fly(0.5);
	else if (key == GLUT_KEY_F4)
        cam.fly(-0.5);
	else if (key == GLUT_KEY_F5)	// z-movement
        cam.walk(0.5);
    else if (key == GLUT_KEY_F6)
        cam.walk(-0.5);

	else if (key == GLUT_KEY_F7)	// yaw (by y-axis)
        cam.yaw(2.5);
	else if (key == GLUT_KEY_F8)
        cam.yaw(-2.5);
	else if (key == GLUT_KEY_F9)	// pitch (by x-axis)
        cam.pitch(2.5);
    else if (key == GLUT_KEY_F10)
        cam.pitch(-2.5);
	else if (key == GLUT_KEY_HOME)	// roll (by z-axis)
        cam.roll(2.5);
    else if (key == GLUT_KEY_END)
        cam.roll(-2.5);

	else if (key == GLUT_KEY_LEFT)	// same as town
        cam.yaw(2.5);
	else if (key == GLUT_KEY_RIGHT)
        cam.yaw(-2.5);
	else if (key == GLUT_KEY_UP)	
        cam.walk(-0.5);
    else if (key == GLUT_KEY_DOWN)
        cam.walk(0.5);

	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);                    
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);  
	glutInitWindowSize(600,600);              
	glutInitWindowPosition(100, 100);         
	glutCreateWindow(argv[0]);                
	glutDisplayFunc(display);                 
	glutReshapeFunc(reshape);                 
	glutKeyboardFunc(keyboard);                
	glutSpecialFunc(specialkey);             
    quadric = gluNewQuadric();
    gluQuadricTexture(quadric, GL_TRUE);

	init();                                   
	glutMainLoop();                           
	return 0;
}

